import os
import collections
from pathlib import Path
from pkg_resources import resource_filename

import yaml

from .typing import Config


def update_config(template: Config, config: Config) -> Config:
    """
    Update a dictionary with key-value pairs found in another one.

    It is a recursive update.

    Parameters
    ----------
    template
        Initial dictionary to update.
    config
        New dictionary to update the original with.

    Returns
    -------
        The resulting (updated) dictionary.
    """
    if not config:
        return template
    for key, value in config.items():
        if isinstance(value, collections.Mapping):
            recurse = update_config(template.get(key, {}), value)
            template[key] = recurse
        else:
            template[key] = config[key]
    return template


def complete_config(config: Config) -> Config:
    """
    Complete configuration with complete paths and parameters.

    Parameters
    ----------
    config
        Input configuration.

    Returns
    -------
        The completed configuration.
    """
    # Allow changing home path through environment variable (for testing)
    home = Path(os.environ.get('MARKDOWNREVEAL_HOME', str(Path.home())))
    config['local_path'] = home / config['local_path']
    config['output_path'] = config['local_path'] / 'out'
    config['reveal_extra']['theme'] = config['theme']
    return config


def load_config() -> Config:
    """
    Load configuration file template.

    Returns
    -------
        The configuration file template.
    """
    # Default Markdownreveal configuration
    config_template = resource_filename(__name__, 'config.template.yaml')
    config = yaml.load(open(config_template))

    # Local configuration (load first for style path)
    local_config = {}
    config_file = Path('config.yaml')
    if config_file.exists():
        local_config = yaml.load(config_file.read_text())
        update_config(config, local_config)
    complete_config(config)

    # Style configuration
    style_config = {}
    config_file = config['output_path'] / 'markdownrevealstyle' / 'config.yaml'
    if config_file.exists():
        style_config = yaml.load(config_file.read_text())
        update_config(config, style_config)
    complete_config(config)

    # Local configuration (override style configuration)
    update_config(config, local_config)
    complete_config(config)

    return config



"""
Setup module.
"""
from setuptools import setup
from markdownreveal import __version__


setup(
    name='markdownreveal',
    version=__version__,
    description='Create presentations with simple Markdown notation',
    long_description='''This tool allows you to create and visualize
        presentations with simple Markdown notation. It is based on
        reveal.js and is able to generate the required HTML and refresh
        your browser view for easier and faster presentation creation.''',
    url='https://github.com/markdownreveal/markdownreveal',
    author='Miguel Sánchez de León Peque',
    author_email='peque@neosit.es',
    license='BSD License',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    keywords='markdown reveal reveal.js presentation',
    entry_points={
        'console_scripts': [
            'mdr = markdownreveal.commands:cli',
            'markdownreveal = markdownreveal.commands:cli',
        ],
    },
    packages=['markdownreveal'],
    package_data={'markdownreveal': ['config.template.yaml']},
    install_requires=[
        'PyYAML',
        'requests',
        'pypandoc',
        'livereload',
        'watchdog',
        'click',
        'click_default_group'],
    extras_require={
        'dev': [],
        'test': ['tox'],
        'docs': ['sphinx', 'numpydoc', 'sphinx_rtd_theme'],
    },
)



import re
from typing import List


def find_indexes(haystack: List[str], regex: str) -> List[int]:
    """
    Find indexes in a list where a regular expression matches.

    Parameters
    ----------
    haystack
        List of strings.
    regex
        Regular expression to match.

    Returns
    -------
        The indexes where the regular expression was found.
    """
    return [i for i, item in enumerate(haystack) if re.search(regex, item)]


def find_style_file(filename, config):
    """
    TODO
    """
    outpath = config['local_path'] / 'out'
    filepath = outpath / config['style_path'] / config[filename]
    if not filepath.exists():
        filepath = outpath / 'markdownrevealstyle' / config[filename]
    if not filepath.exists():
        return
    return filepath.relative_to(filepath.parents[1])


def tweak_html_footer(html, footer):
    """
    TODO
    """
    if not footer:
        return False
    text = '<div class="footer">%s</div>' % footer
    for index in find_indexes(html, '<div class=\"reveal\">'):
        html.insert(index + 1, text)
    return True


def tweak_html_header(html, header):
    """
    TODO
    """
    if not header:
        return
    text = '<div class="header">%s</div>' % header
    for index in find_indexes(html, '<div class=\"reveal\">'):
        html.insert(index + 1, text)


def tweak_html_warmup(html, config):
    """
    TODO
    """
    fname = find_style_file('style_warmup', config)
    if not fname:
        return
    text = '<section><img src="%s" /></section>' % fname
    index = find_indexes(html, 'div class="slides"')[0]
    html.insert(index + 1, text)


def tweak_html_logo(html, config):
    """
    TODO
    """
    fname = find_style_file('style_logo', config)
    if not fname:
        return
    text = '<div class="logo"><img src="%s" /></div>' % fname
    for index in find_indexes(html, '<div class=\"reveal\">'):
        html.insert(index + 1, text)


def tweak_html_background(html, config):
    """
    TODO
    """
    fname = find_style_file('style_background', config)
    if not fname:
        return
    text = '<section data-background="%s">' % fname
    for index in find_indexes(html, '<section>'):
        html[index] = html[index].replace('<section>', text)


def tweak_html_css(html, config):
    """
    TODO
    """
    fname = find_style_file('style_custom_css', config)
    if not fname:
        return
    index = find_indexes(html, 'stylesheet.*id="theme"')[0]
    text = '<link rel="stylesheet" href="%s">' % fname
    html.insert(index + 1, text)
    return True


def tweak_html(html, config):
    """
    TODO
    """
    html = html.splitlines()
    # TODO: footer and header could be added to the style template (i.e.:
    #       within a config.yaml file part of the template which Markdownreveal
    #       should be able to load and override if necessary)
    tweak_html_footer(html, config['footer'])
    tweak_html_header(html, config['header'])
    tweak_html_warmup(html, config)
    tweak_html_logo(html, config)
    tweak_html_background(html, config)
    tweak_html_css(html, config)
    return '\n'.join(html)
